
#############################################################
# This file was automatically generated on 2014-01-14.      #
#                                                           #
# Bindings Version 2.0.0                                    #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generator git on tinkerforge.com                   #
#############################################################

package BrickletPiezoBuzzer;
=comment
        Device for controlling a piezo buzzer
=cut

use Tinkerforge::Device;
use strict;
use warnings;
use Carp;
use threads;
use threads::shared;

use constant DEVICE_IDENTIFIER => 214;
use constant CALLBACK_BEEP_FINISHED => 3;
use constant CALLBACK_MORSE_CODE_FINISHED => 4;

use constant FUNCTION_BEEP => 1;
use constant FUNCTION_MORSE_CODE => 2;
use constant FUNCTION_GET_IDENTITY => 255;


sub new
{
=comment
        Creates an object with the unique device ID *uid* and adds it to
        the IP Connection *ipcon*.
=cut
    my ($class, $host, $port) = @_;

    my $self :shared = shared_clone({super => shared_clone(new Device($host,$port)),
                                     api_version => [2, 0, 0],
                                     response_expected => shared_clone({&FUNCTION_BEEP => Device->RESPONSE_EXPECTED_FALSE,
                                                                        &FUNCTION_MORSE_CODE => Device->RESPONSE_EXPECTED_FALSE,
                                                                        &CALLBACK_BEEP_FINISHED => Device->RESPONSE_EXPECTED_ALWAYS_FALSE,
                                                                        &CALLBACK_MORSE_CODE_FINISHED => Device->RESPONSE_EXPECTED_ALWAYS_FALSE,
                                                                        &FUNCTION_GET_IDENTITY => Device->RESPONSE_EXPECTED_ALWAYS_TRUE}),
                                    callback_formats => shared_clone({&CALLBACK_BEEP_FINISHED => '',
                                                                      &CALLBACK_MORSE_CODE_FINISHED => ''})});

    $self->{super}->{ipcon}->{devices}->{$self->{super}->{uid}} = $self;

    $self->{super}->{api_version} = $self->{api_version};

    bless($self, $class);

    return $self;
}

sub beep
{
=comment
        Beeps with the duration in ms. For example: If you set a value of 1000,
        the piezo buzzer will beep for one second.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $duration) = @_;

    $self->{super}->send_request($self, &FUNCTION_BEEP, [$duration], 'L', '');
}

sub morse_code
{
=comment
        Sets morse code that will be played by the piezo buzzer. The morse code
        is given as a string consisting of "." (dot), "-" (minus) and " " (space)
        for *dits*, *dahs* and *pauses*. Every other character is ignored.
        
        For example: If you set the string "...---...", the piezo buzzer will beep
        nine times with the durations "short short short long long long short 
        short short".
        
        The maximum string size is 60.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $morse) = @_;

    $self->{super}->send_request($self, &FUNCTION_MORSE_CODE, [$morse], 'Z60', '');
}

sub get_identity
{
=comment
        Returns the UID, the UID where the Bricklet is connected to, 
        the position, the hardware and firmware version as well as the
        device identifier.
        
        The position can be 'a', 'b', 'c' or 'd'.
        
        The device identifiers can be found :ref:`here <device_identifier>`.
        
        .. versionadded:: 2.0.0~(Plugin)
=cut
    lock($Device::DEVICE_LOCK);

    my ($self) = @_;

    return $self->{super}->send_request($self, &FUNCTION_GET_IDENTITY, [], '', 'Z8 Z8 a C3 C3 S');
 }


sub register_callback
{
=comment
        Registers a callback with ID *id* to the function *callback*.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $id, $callback) = @_;

    $self->{super}->{registered_callbacks}->{$id} = '&'.caller.'::'.$callback;
}

sub get_api_version
{
=comment
        Returns the API version (major, minor, revision) of the bindings for
        this device.
=cut
    my ($self) = @_;

    return $self->{super}->{api_version};
}

sub get_response_expected
{
=comment
        Returns the response expected flag for the function specified by the
        *function_id* parameter. It is *true* if the function is expected to
        send a response, *false* otherwise.

        For getter functions this is enabled by default and cannot be disabled,
        because those functions will always send a response. For callback
        configuration functions it is enabled by default too, but can be
        disabled via the set_response_expected function. For setter functions
        it is disabled by default and can be enabled.

        Enabling the response expected flag for a setter function allows to
        detect timeouts and other error conditions calls of this setter as
        well. The device will then send a response for this purpose. If this
        flag is disabled for a setter function then no response is send and
        errors are silently ignored, because they cannot be detected.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $function_id) = @_;

    if(defined($self->{response_expected}->{$function_id}))
    {
        return $self->{response_expected}->{$function_id};
    }
    else
    {
        croak('Unknown function');
    }
}

sub set_response_expected
{
=comment
        Changes the response expected flag of the function specified by the
        *function_id* parameter. This flag can only be changed for setter
        (default value: *false*) and callback configuration functions
        (default value: *true*). For getter functions it is always enabled
        and callbacks it is always disabled.

        Enabling the response expected flag for a setter function allows to
        detect timeouts and other error conditions calls of this setter as
        well. The device will then send a response for this purpose. If this
        flag is disabled for a setter function then no response is send and
        errors are silently ignored, because they cannot be detected.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $function_id, $response_expected) = @_;

    if(defined($self->{response_expected}->{$function_id}) &&
      ($self->{response_expected}->{$function_id} != Device->RESPONSE_EXPECTED_ALWAYS_TRUE ||
       $self->{response_expected}->{$function_id} != Device->RESPONSE_EXPECTED_ALWAYS_FALSE))
    {
       if($response_expected == Device->RESPONSE_EXPECTED_TRUE ||
          $response_expected == Device->RESPONSE_EXPECTED_FALSE)
       {
           $self->{response_expected}->{$function_id} = $response_expected;
       }
       else
       {
           croak('Unknown value');
       }
    }
    else
    {
        croak('Unknown function or value');
    }
}

sub set_response_expected_all
{
=comment
        Changes the response expected flag for all setter and callback
        configuration functions of this device at once.
=cut
    lock($Device::DEVICE_LOCK);

    my ($self, $response_expected) = @_;

    foreach my $key (sort keys $self->{response_expected})
    {
        if($response_expected == Device->RESPONSE_EXPECTED_TRUE ||
           $response_expected == Device->RESPONSE_EXPECTED_FALSE)
        {
            if($self->{response_expected}->{$key} != Device->RESPONSE_EXPECTED_ALWAYS_TRUE ||
               $self->{response_expected}->{$key} != Device->RESPONSE_EXPECTED_ALWAYS_FALSE)
            {
                $self->{response_expected}->{$key} = $response_expected;
            }
        }
        else
        {
            croak('Unknown value');
        }
    }
}

1;
